from flask import Blueprint, request
from flask_restful import Api, Resource, abort
from marshmallow import ValidationError

from portal.database.models import Course
from portal.rest import CourseSchema, delete_entity, write_entity
from portal.tools.logging import log


courses = Blueprint('courses', __name__, url_prefix='/courses')
courses_api = Api(courses)


course_schema = CourseSchema()
courses_schema = CourseSchema(many=True, only=('id', 'name', 'codename'))


class CourseResource(Resource):
    def get(self, cid):
        log.info(f"GET to {request.url}")
        course = Course.query.filter_by(id=cid).first()
        if not course:
            log.debug(f"Did not find course with id={cid}. Assuming search by codename was intended.")
        course = Course.query.filter_by(codename=cid).first()
        if not course:
            log.debug(f"Did not find course with codename={cid}. Aborting request processing.")
            abort(404, message="Course with codename={} not found.".format(cid))
        return course_schema.dump(course)

    def delete(self, cid):
        log.info(f"DELETE to {request.url}")
        course = Course.query.filter_by(id=cid).first()
        if not course:
            log.debug(f"Did not find course with id={cid}; assuming search by codename was intended")
        course = Course.query.filter_by(codename=cid).first()
        if not course:
            log.debug(f"Did not find course with codename={cid}. Aborting request processing.")
            abort(404, message="Course with codename={} not found.".format(cid))
        delete_entity(course)
        return '', 204

    def put(self, cid):
        log.info(f"PUT to {request.url}")
        course = Course.query.filter_by(id=cid).first()
        if not course:
            log.debug(f"Did not find course with id={cid}; assuming search by codename was intended")
        course = Course.query.filter_by(codename=cid).first()
        if not course:
            log.debug(f"Did not find course with codename={cid}. Aborting request processing.")
            abort(404, message="Course with codename={} not found.".format(cid))

        json_data = request.get_json()
        if not json_data:
            abort(400, message='No data provided for course update.')
        try:
            data = course_schema.load(json_data)
        except ValidationError as err:
            log.info(f"Validation failed on: {err.messages}")
            return abort(422, err.messages)  # TODO: flash, process errors

        course.name = data['name']
        course.codename = data['codename']
        # relationships are written elsewhere
        write_entity(course)
        log.debug(f"Course update successful. Course={course}")
        return '', 204


class CourseList(Resource):
    def get(self):
        log.info(f"GET to {request.url}")
        courses = Course.query.all()
        return courses_schema.dump(courses)

    def post(self):
        log.info(f"POST to {request.url}")
        json_data = request.get_json()
        if not json_data:
            abort(400, message='No data provided for course update.')
        try:
            data = course_schema.load(json_data)
        except ValidationError as err:
            log.info(f"Validation failed on: {err.messages}")
            return abort(422, err.messages)  # TODO: flash, process errors
        name = data[0]['name']
        codename = data[0]['codename']
        new_course = Course(name=name, codename=codename)
        write_entity(new_course)
        log.debug(f"Created course={new_course}")
        return course_schema.dump(new_course)[0], 201


class CourseNotesToken(Resource):
    def get(self, cid):
        course = Course.query.filter_by(id=cid).first()
        if not course:
            log.debug(f"Did not find course with id={cid}; assuming search by codename was intended")
        course = Course.query.filter_by(codename=cid).first()
        if not course:
            log.debug(f"Did not find course with codename={cid}. Aborting request processing.")
            abort(404, message=f"Course with codename={cid} not found.")
        return course.notes_access_token

    def put(self, cid):
        course = Course.query.filter_by(id=cid).first()
        if not course:
            log.debug(f"Did not find course with id={cid}; assuming search by codename was intended")
        course = Course.query.filter_by(codename=cid).first()
        if not course:
            log.debug(f"Did not find course with codename={cid}. Aborting request processing.")
            abort(404, message=f"Course with codename={cid} not found.")

        json_data = request.get_json()
        if not json_data:
            abort(400, message='No data provided for notes token update.')

        course.notes_access_token = json_data  # TODO: validation
        write_entity(course)
        log.info(f"Updated notes access token for course id={cid} to {json_data}")
        return '', 204


courses_api.add_resource(CourseResource, '/<string:cid>')
courses_api.add_resource(CourseList, '/')
courses_api.add_resource(CourseNotesToken, "/<string:cid>/notes_access_token")
